From: Tejun Heo <[email protected]>

commit addf0d2fbf84fa527e26af8ae670f7d85cc6c375 from
https://github.com/altera-opensource/linux-socfpga.git

[upstream commit 1756d7994ad85c2479af6ae5a9750b92324685af]

cgroup process migration permission checks are performed at write time as
whether a given operation is allowed or not is dependent on the content of
the write - the PID. This currently uses current's credentials which is a
potential security weakness as it may allow scenarios where a less
privileged process tricks a more privileged one into writing into a fd that
it created.

This patch makes both cgroup2 and cgroup1 process migration interfaces to
use the credentials saved at the time of open (file->f_cred) instead of
current's.

Reported-by: "Eric W. Biederman" <[email protected]>
Suggested-by: Linus Torvalds <[email protected]>
Fixes: 187fe84067bd ("cgroup: require write perm on common ancestor when moving 
processes on the default hierarchy")
Reviewed-by: Michal Koutný <[email protected]>
Signed-off-by: Tejun Heo <[email protected]>
Signed-off-by: Wenlin Kang <[email protected]>
---
 kernel/cgroup/cgroup-v1.c | 7 ++++---
 kernel/cgroup/cgroup.c    | 8 ++++++++
 2 files changed, 12 insertions(+), 3 deletions(-)

diff --git a/kernel/cgroup/cgroup.c b/kernel/cgroup/cgroup.c
index 7173e8b5f5a1..684c16849eff 100644
--- a/kernel/cgroup/cgroup.c
+++ b/kernel/cgroup/cgroup.c
@@ -4856,9 +4856,16 @@ static ssize_t cgroup_procs_write(struct 
kernfs_open_file *of,
        src_cgrp = task_cgroup_from_root(task, &cgrp_dfl_root);
        spin_unlock_irq(&css_set_lock);
 
+       /*
+        * Process and thread migrations follow same delegation rule. Check
+        * permissions using the credentials from file open to protect against
+        * inherited fd attacks.
+        */
+       saved_cred = override_creds(of->file->f_cred);
        ret = cgroup_attach_permissions(src_cgrp, dst_cgrp,
                                        of->file->f_path.dentry->d_sb, true,
                                        ctx->ns);
+       revert_creds(saved_cred);
        if (ret)
                goto out_finish;
 
@@ -4883,6 +4890,7 @@ static ssize_t cgroup_threads_write(struct 
kernfs_open_file *of,
        struct cgroup_file_ctx *ctx = of->priv;
        struct cgroup *src_cgrp, *dst_cgrp;
        struct task_struct *task;
+       const struct cred *saved_cred;
        ssize_t ret;
        bool locked;
 
diff --git a/kernel/cgroup/cgroup-v1.c b/kernel/cgroup/cgroup-v1.c
index 4296e0e581cc..433b9e840b38 100644
--- a/kernel/cgroup/cgroup-v1.c
+++ b/kernel/cgroup/cgroup-v1.c
@@ -507,10 +507,11 @@ static ssize_t __cgroup1_procs_write(struct 
kernfs_open_file *of,
                goto out_unlock;
 
        /*
         * Even if we're attaching all tasks in the thread group, we only
         * need to check permissions on one of them.
         */
-       cred = current_cred();
+       cred = of->file->f_cred;
        tcred = get_task_cred(task);
        if (!uid_eq(cred->euid, GLOBAL_ROOT_UID) &&
            !uid_eq(cred->euid, tcred->uid) &&
@@ -507,10 +507,11 @@ static ssize_t __cgroup1_procs_write(struct 
 
        /*
-        * Even if we're attaching all tasks in the thread group, we only
-        * need to check permissions on one of them.
+        * Even if we're attaching all tasks in the thread group, we only need
+        * to check permissions on one of them. Check permissions using the
+        * credentials from file open to protect against inherited fd attacks.
         */
-- 
2.25.1

-=-=-=-=-=-=-=-=-=-=-=-
Links: You receive all messages sent to this group.
View/Reply Online (#12584): 
https://lists.yoctoproject.org/g/linux-yocto/message/12584
Mute This Topic: https://lists.yoctoproject.org/mt/98921505/21656
Group Owner: [email protected]
Unsubscribe: https://lists.yoctoproject.org/g/linux-yocto/unsub 
[[email protected]]
-=-=-=-=-=-=-=-=-=-=-=-

Reply via email to